/*	$OpenBSD: conf.h,v 1.1 2013/09/04 14:38:26 patrick Exp $	*/
/*	$NetBSD: conf.h,v 1.8 2002/02/10 12:26:03 chris Exp $	*/

#ifndef _MACHINE_CONF_H_
#define	_MACHINE_CONF_H_

#include <sys/conf.h>

/*
 * BEAGLEBOARD specific device includes go in here
 */

/*#define	CONF_HAVE_GPIO*/

#include <arm/conf.h>

#endif	/* _MACHINE_CONF_H_ */
